/*
 * Daneel - Dalvik to Java bytecode compiler
 * Copyright (C) 2011  IcedRobot team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * This file is subject to the "Classpath" exception:
 *
 * Linking this library statically or dynamically with other modules is
 * making a combined work based on this library.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent
 * modules, and to copy and distribute the resulting executable under terms
 * of your choice, provided that you also meet, for each linked independent
 * module, the terms and conditions of the license of that module.  An
 * independent module is a module which is not derived from or based on
 * this library.  If you modify this library, you may extend this exception
 * to your version of the library, but you are not obligated to do so.  If
 * you do not wish to do so, delete this exception statement from your
 * version.
 */

package org.icedrobot.daneel.rewriter;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertSame;

import java.lang.annotation.Annotation;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import org.icedrobot.daneel.DexifyingRunner;
import org.junit.Test;
import org.junit.runner.RunWith;

@RunWith(DexifyingRunner.class)
public class AnnotationTest {

    @Test
    public void testClassAnnotation() {
        Annotation[] a = DEXCode.class.getAnnotations();
        assertEquals(1, a.length);
        assertSame(TestAnnotationA.class, a[0].annotationType());
    }

    @Test
    public void testFieldAnnotation() throws Exception {
        Annotation[] a = DEXCode.class.getField("field").getAnnotations();
        assertEquals(1, a.length);
        assertSame(TestAnnotationB.class, a[0].annotationType());
    }

    @Test
    public void testMethodAnnotation() throws Exception {
        Annotation[] a = DEXCode.class.getMethod("method").getAnnotations();
        assertEquals(1, a.length);
        assertSame(TestAnnotationC.class, a[0].annotationType());
    }

    @Retention(RetentionPolicy.RUNTIME)
    @Target({ ElementType.TYPE })
    private static @interface TestAnnotationA {
    };

    @Retention(RetentionPolicy.RUNTIME)
    @Target({ ElementType.FIELD })
    private static @interface TestAnnotationB {
    };

    @Retention(RetentionPolicy.RUNTIME)
    @Target({ ElementType.METHOD })
    private static @interface TestAnnotationC {
    };

    // Keep this class named "DEXCode" to push it through Daneel.
    @TestAnnotationA
    public static class DEXCode {

        @TestAnnotationB
        public int field;

        @TestAnnotationC
        public void method() {
        }
    };
}
